import { container, T_PG } from "#/di";
import { WapiV3GuildOverview } from "#/lib/types";
import { WApi } from "#/lib/wapi";
import { type } from "arktype";
import {parseDate} from "chrono-node";

export async function update_all_guilds() {
  const api = new WApi()
  const ans = await api.get('/v3/guild/list/guild')
  if(ans.status !== 200){
    throw new Error('Failed to get guild list from wapi')
  }
  const parsed = type({
    "[string]": {
      uuid: "string",
      prefix: "string",
    }
  }).assert(ans.data)

  const db = container.get(T_PG)
  await db.begin(async (sql) => {
    for(const [guild_name, guild] of Object.entries(parsed)){
      await sql`insert into wynn_guild_info
        (uid, name, prefix)
      values
        (${guild.uuid}, ${guild_name}, ${guild.prefix})
        on conflict (uid) do update set
        name = EXCLUDED.name,
        prefix = EXCLUDED.prefix
      `
    }
  })
}

export async function update_guild({
  guild_name
}:{
    guild_name: string
  }) {
  const api = new WApi()

  const ans = await api.get(`/v3/guild/${guild_name}`)
  if(ans.status !== 200){
    throw new Error('Failed to get guild into from wapi')
  }
  const parsed = WapiV3GuildOverview.assert(ans.data)

  const db = container.get(T_PG)

  await db.begin(async (sql) => {
    await sql`insert into wynn_guild_info
      (uid, name, prefix, level, xp_percent, territories, wars, created)
    values
      (${parsed.uuid}, ${parsed.name}, ${parsed.prefix}, ${parsed.level}, ${parsed.xpPercent}, ${parsed.territories}, ${parsed.wars}, ${parseDate(parsed.created)})
      on conflict (uid) do update set
      name = EXCLUDED.name,
      prefix = EXCLUDED.prefix,
      level = EXCLUDED.level,
      xp_percent = EXCLUDED.xp_percent,
      territories = EXCLUDED.territories,
      wars = EXCLUDED.wars,
      created = EXCLUDED.created
    `
    const {total, ...rest} = parsed.members

    for(const [rank_name, rank] of Object.entries(rest)){
      for(const [userName, member] of Object.entries(rank)) {
        await sql`insert into wynn_guild_members
          (guild_id, member_id, rank, joined_at, contributed)  values
          (${parsed.uuid}, ${member.uuid}, ${rank_name}, ${parseDate(member.joined)}, ${member.contributed})
          on conflict (guild_id, member_id) do update set
          rank = EXCLUDED.rank,
          joined_at = EXCLUDED.joined_at,
          contributed = EXCLUDED.contributed
        `

        await sql`insert into minecraft_user
          (uid, name, server) values (${member.uuid}, ${userName}, ${member.server})
          on conflict (uid) do update set
          name = EXCLUDED.name,
          server = EXCLUDED.server
        `
      }
    }
    for (const [season, seasonData] of Object.entries(parsed.seasonRanks)) {
      await sql`insert into wynn_guild_season_results
        (guild_id, season, rating, territories)  values
        (${parsed.uuid}, ${season}, ${seasonData.rating}, ${seasonData.finalTerritories})
        on conflict (guild_id, season) do update set
        rating = EXCLUDED.rating,
        territories = EXCLUDED.territories
      `
    }
  })
}
