import { Sql } from "postgres";
import { CreateMessageOptions, Embed, InteractionCallbackOptions } from "discordeno"
import {type} from "arktype"
import { TabWriter } from "#/lib/tabwriter"
import * as md from 'ts-markdown-builder';


export const formGuildOnlineMessage = async (guild_id: string, sql:Sql): Promise<CreateMessageOptions & InteractionCallbackOptions> => {
  const result = await sql`select
    name,
    rank,
    contributed,
    minecraft_user.server as server
    from wynn_guild_members inner join minecraft_user
    on minecraft_user.uid = wynn_guild_members.member_id
    where minecraft_user.server is not null
    and wynn_guild_members.guild_id = ${guild_id}
  `
  const members = type({
    name: "string",
    rank: "string",
    contributed: "string",
    server: "string",
  }).array().assert(result)

  if(members.length == 0) {
    return {
      content: "nobody is online :(",
    }
  }

  members.sort((a, b) => Number(b.contributed) - Number(a.contributed))

  // group members by server
  const membersByServer= members.reduce((acc, member) => {
    if(acc[member.server] == undefined) {
      acc[member.server] = []
    }
    acc[member.server].push(member)
    return acc
  }, {} as Record<string, typeof members>)


  const output = Object.entries(membersByServer).map(([server, mx]) => {
    return `**[${server}]** (${mx.length}): ${mx.map(m => m.name).join(", ")}`
  }).join(", ")


  return {
    content: `**total**: ${members.length} \n` + output,
  }
}

export const formGuildLeaderboardMessage = async (guild_id: string, sql:Sql): Promise<CreateMessageOptions & InteractionCallbackOptions> => {
  const result = await sql`select
    name,
    rank,
    contributed
    from wynn_guild_members inner join minecraft_user
    on minecraft_user.uid = wynn_guild_members.member_id
    where wynn_guild_members.guild_id = ${guild_id}
  `
  const members = type({
    name: "string",
    rank: "string",
    contributed: "string",
  }).array().assert(result)

  const tw = new TabWriter()
  members.sort((a, b) => Number(b.contributed) - Number(a.contributed))
  let idx = 1
  for (const member of members.slice(0,10)) {
    tw.add([
      `${idx}.`,
      member.rank,
      member.name,
      Number(member.contributed).toLocaleString(),
    ])
    idx = idx + 1
  }

  const built = tw.build()
  const output = [
    md.heading("Guild Exp:"),
    md.codeBlock(built),
  ].join("\n\n")
  return {
    content: output,
  }
}
