import {bot} from "#/bot"
import { ActivityTypes, ApplicationCommandOptionTypes, InteractionTypes } from "discordeno"
import { InteractionHandler, MuxHandler, SlashHandler } from "./types"
import { root } from "./slash_commands"
import { uuid4 } from "@temporalio/workflow"


export const slashHandler: InteractionHandler = async (interaction) => {
  if(!interaction.data) {
    return
  }
  if(!interaction.data.name) {
    return
  }
  const commandPath: string[] = [interaction.data.name]
  if(interaction.data.options) {
    for(const option of interaction.data.options) {
      if(option.type === ApplicationCommandOptionTypes.SubCommand) {
        commandPath.push(option.name)
      }
    }
  }

  let rootHandler: SlashHandler = root

  let cur: SlashHandler | MuxHandler<any> = rootHandler
  for(let i = 0; i < commandPath.length; i++) {
    const handlerName = commandPath[i]
    if(typeof cur === 'function') {
      return interaction.respond(`oh no a bug. router: got function instead of next handler`)
    }
    let next: SlashHandler | MuxHandler<any> | undefined = cur[handlerName]
    if(!cur) {
      return interaction.respond(`command not implemented at ${handlerName}`, {isPrivate: true})
    }
    cur = next
  }
  if(typeof cur === 'function') {
    // yay, we found command with handler, run the command
    try {
      await cur(interaction)
      return
    }catch(e) {
      const errId = uuid4()
      console.error("invokation exception",errId, commandPath, e)
      return interaction.respond(`invokation exception: ${errId}`, {isPrivate: true})
    }
  }
  // ok now we need to go down the handler tree.
}

export const events = {
  interactionCreate: async (interaction) => {
    if(interaction.acknowledged) {
      return
    }
    if(interaction.type !== InteractionTypes.ApplicationCommand) {
      return
    }
    await slashHandler(interaction)
    return
  },
  ready: async ({shardId}) => {
    await bot.gateway.editShardStatus(shardId, {
      status: 'online',
      activities: [
        {
          name: 'im frog',
          type: ActivityTypes.Playing,
          timestamps: {
            start: Date.now(),
          },
        },
      ],
    })
  }
} as typeof bot.events
