import { type } from "arktype"

export const WynnGuildOverviewMember = type({
  uuid: "string",
  online: "boolean",
  server: "null | string",
  contributed: "number",
  contributionRank: "number",
  joined: "string"
})

const WapiV3GuildMembers = type({
  "[string]": WynnGuildOverviewMember
})

export const WapiV3GuildOverview = type({
  uuid: "string",
  name: "string",
  prefix: "string",
  level: "number",
  xpPercent: "number",
  territories: "number",
  wars: "number",
  created: "string",
  members: {
    total: "number",
    owner: WapiV3GuildMembers,
    chief: WapiV3GuildMembers,
    strategist: WapiV3GuildMembers,
    captain: WapiV3GuildMembers,
    recruiter: WapiV3GuildMembers,
    recruit: WapiV3GuildMembers,
  },
  online: "number",
  banner: {
    base: "string",
    tier: "number",
    structure: "string",
    layers: type({ colour: "string", pattern: "string" }).array(),
  },
  seasonRanks: {
    "[string]": {
      rating: "number",
      finalTerritories: "number"
    }
  }
})

const WynnItemRarity = type.enumerated("common", "fabled", "legendary", "mythic", "rare", "set", "unique")

const WynnSkills = type.enumerated(
  "alchemism",
  "armouring",
  "cooking",
  "jeweling",
  "scribing",
  "tailoring",
  "weaponsmithing",
  "woodworking",
)

const WynnDropMeta = type("object")

const WynnDropRestriction = type.enumerated("normal", "never", "dungeon", "lootchest")

const WynnItemRestrictions = type.enumerated("untradable", "quest item")

const WynnEquipRequirements = type({
  level: "number",
  "classRequirement?": "string",
  "intelligence?": "number",
  "strength?": "number",
  "dexterity?": "number",
  "defence?": "number",
  "agility?": "number",
  "skills?": WynnSkills.array(),
})

const WynnBaseStats = type({
  "[string]": type("number").or({
    min: "number",
    raw: "number",
    max: "number",
  })
})


const WynnIdentifications = type({
  "[string]": type("number").or({
    min: "number",
    raw: "number",
    max: "number",
  })
})

const WynnItemIcon = type({
  format: "string",
  value: "unknown"
})

export const WapiV3ItemTool = type({
  internalName: "string",
  type: '"tool"',
  toolType: type.enumerated("axe", "pickaxe", "rod", "scythe"),
  "identified?": "boolean",
  gatheringSpeed: "number",
  requirements: {
    level: "number",
  },
  icon: WynnItemIcon,
  rarity: WynnItemRarity
})

export const WapiV3ItemTome = type({
  internalName: "string",
  tomeType: type.enumerated(
    "guild_tome",
    "marathon_tome",
    "mysticism_tome",
    "weapon_tome",
    "armour_tome",
    "expertise_tome",
    "lootrun_tome",
  ),
  raidReward: "boolean",
  type: '"tome"',
  "restrictions?": WynnItemRestrictions,
  "dropMeta?": WynnDropMeta,
  dropRestriction: WynnDropRestriction,
  requirements: WynnEquipRequirements,
  "lore?": "string",
  icon: WynnItemIcon,
  "base?": WynnBaseStats,
  rarity: WynnItemRarity
})

export const WapiV3ItemCharm = type({
  internalName: "string",
  type: '"charm"',
  "restrictions?": WynnItemRestrictions,
  "dropMeta?": WynnDropMeta,
  dropRestriction: WynnDropRestriction,
  requirements: WynnEquipRequirements,
  "lore?": "string",
  icon: WynnItemIcon,
  base: WynnBaseStats,
  rarity: WynnItemRarity
})

export const WapiV3ItemAccessory = type({
  internalName: "string",
  type: '"accessory"',
  "identified?": "boolean",
  accessoryType: type.enumerated("ring", "necklace", "bracelet"),
  "majorIds?": {
    "[string]": "string"
  },
  "restrictions?": WynnItemRestrictions,
  "dropMeta?": WynnDropMeta,
  dropRestriction: WynnDropRestriction,
  requirements: WynnEquipRequirements,
  "lore?": "string",
  icon: WynnItemIcon,
  "identifications?": WynnIdentifications,
  "base?": WynnBaseStats,
  rarity: WynnItemRarity
})

export const WapiV3ItemIngredient = type({
  internalName: "string",
  type: '"ingredient"',
  requirements: {
    level: "number",
    skills: WynnSkills.array(),
  },
  icon: WynnItemIcon,
  "identifications?": WynnIdentifications,
  tier: "number",
  consumableOnlyIDs: {
    "[string]": "number"
  },
  ingredientPositionModifiers: {
    "[string]": "number"
  },
  itemOnlyIDs: {
    "[string]": "number"
  },
  "droppedBy?": type({
    name: "string",
    coords: type("boolean | null")
    .or(type("number[] == 4"))
    .or(type("number[] == 4").array())
  }).array()
})

export const WapiV3ItemMaterial = type({
  internalName: "string",
  type: '"material"',
  identified: "boolean",
  requirements: {
    level: "number",
  },
  craftable: type.enumerated(
    "potions", "food", "scrolls",
    "helmets", "chestplates", "rings", "bracelets",
    "necklaces", "boots", "leggings", "bows", "wands", "spears",
    "daggers", "chestplates", "helmets"
  ).array(),
  icon: WynnItemIcon,
  tier: "number"
})

export const WapiV3ItemWeapon = type({
  internalName: "string",
  type: '"weapon"',
  "identified?": "boolean",
  "allowCraftsman?": "boolean",
  weaponType: type.enumerated("bow", "relik", "wand", "dagger", "spear"),
  attackSpeed: type.enumerated(
    "super_slow", "very_slow", "slow", "normal", "fast", "very_fast", "super_fast"
  ),
  "powderSlots?": "number",
  "averageDps?": "number",
  "restrictions?": WynnItemRestrictions,
  "dropMeta?": WynnDropMeta,
  "dropRestriction?": WynnDropRestriction,
  requirements: WynnEquipRequirements,
  "majorIds?": {
    "[string]": "string"
  },
  "lore?": "string",
  icon: WynnItemIcon,
  "identifications?": WynnIdentifications,
  "base?": WynnBaseStats,
  rarity: WynnItemRarity
})

export const WapiV3ItemArmour = type({
  internalName: "string",
  type: '"armour"',
  armourType: "string",
  "armourMaterial?": "string",
  "armourColor?": "string",
  "identified?": "boolean",
  "allowCraftsman?": "boolean",
  "restrictions?": WynnItemRestrictions,
  dropRestriction: WynnDropRestriction,
  "dropMeta?": WynnDropMeta,
  "icon?": WynnItemIcon,
  requirements: WynnEquipRequirements,
  "majorIds?": {
    "[string]": "string"
  },
  "powderSlots?": "number",
  "lore?": "string",
  "identifications?": WynnIdentifications,
  "base?": WynnBaseStats,
  rarity: WynnItemRarity
})

export const WApiV3Item = WapiV3ItemMaterial
.or(WapiV3ItemWeapon)
.or(WapiV3ItemArmour)
.or(WapiV3ItemIngredient)
.or(WapiV3ItemAccessory)
.or(WapiV3ItemCharm)
.or(WapiV3ItemTome)
.or(WapiV3ItemTool)

export const WApiV3ItemDatabase = type({
  "[string]": WApiV3Item
})
