import { z } from 'zod'

export const WynnGuildOverviewMember = z.object({
  uuid: z.string(),
  online: z.boolean(),
  server: z.null(),
  contributed: z.number(),
  contributionRank: z.number(),
  joined: z.string()
})


const WapiV3GuildMembers = z.record(z.string(), WynnGuildOverviewMember)

export const WapiV3GuildOverview = z.object({
  uuid: z.string(),
  name: z.string(),
  prefix: z.string(),
  level: z.number(),
  xpPercent: z.number(),
  territories: z.number(),
  wars: z.number(),
  created: z.string(),
  members: z.object({
    total: z.number(),
    owner: WapiV3GuildMembers,
    chief: WapiV3GuildMembers,
    strategist: WapiV3GuildMembers,
    captain: WapiV3GuildMembers,
    recruiter: WapiV3GuildMembers,
    recruit: WapiV3GuildMembers,
  }),
  online: z.number(),
  banner: z.object({
    base: z.string(),
    tier: z.number(),
    structure: z.string(),
    layers: z.array(z.object({ colour: z.string(), pattern: z.string() }))
  }),
  seasonRanks: z.record(z.string(),z.object({ rating: z.number(), finalTerritories: z.number() }))
})



const WynnItemRarity = z.enum([
  "common","fabled","legendary","mythic","rare","set","unique",
])
const WynnDropMeta = z.any()
const WynnDropRestriction = z.enum(["normal","never","dungeon", "lootchest"])

const WynnItemRestrictions = z.enum([
  "untradable", "quest item",
])

const WynnEquipRequirements = z.object({
  level: z.number(),
  classRequirement: z.string().optional(),
  intelligence: z.number().optional(),
  strength: z.number().optional(),
  dexterity: z.number().optional(),
  defence: z.number().optional(),
  agility: z.number().optional(),
})

const WynnBaseStats = z.record(z.string(),z.union([
  z.number(),
  z.object({
    min: z.number(),
    raw: z.number(),
    max: z.number(),
  })
]))

const WynnIdentifications = z.record(z.string(), z.union([
  z.number(),
  z.object({
    min: z.number(),
    raw: z.number(),
    max: z.number(),
  })
]))

const WynnItemIcon = z.object({
  format: z.string(),
  value: z.any(),
})

/*
  "Gathering Axe T11": {
    "internalName": "Gathering Axe 11",
    "type": "tool",
    "toolType": "axe",
    "gatheringSpeed": 275,
    "identified": true,
    "requirements": {
      "level": 95
    },
    "icon": {
      "format": "attribute",
      "value": {
        "id": "minecraft:iron_horse_armor",
        "customModelData": "50",
        "name": "gatheringTool.axe11"
      }
    },
    "rarity": "common"
  },
*/
export const WapiV3ItemTool = z.object({
  internalName: z.string(),
  type: z.literal('tool'),
  toolType: z.enum([
    "axe","pickaxe","rod","scythe",
  ]),
  identified: z.boolean().optional(),
  gatheringSpeed: z.number(),
  requirements: z.object({
    level: z.number(),
  }),
  icon: WynnItemIcon,
  rarity:WynnItemRarity,
})

export const WapiV3ItemTome = z.object({
  internalName: z.string(),
  tomeType: z.enum([
    "guild_tome",
    "marathon_tome",
    "mysticism_tome",
    "weapon_tome",
    "armour_tome",
    "expertise_tome",
    "lootrun_tome",
  ]),
  raidReward: z.boolean(),
  type: z.literal('tome'),
  restrictions: WynnItemRestrictions.optional(),
  dropMeta: WynnDropMeta.optional(),
  dropRestriction: WynnDropRestriction,
  requirements: WynnEquipRequirements,
  lore: z.string().optional(),
  icon: WynnItemIcon,
  base: WynnBaseStats.optional(),
  rarity:WynnItemRarity,
})

export const WapiV3ItemCharm = z.object({
  internalName: z.string(),
  type: z.literal('charm'),
  restrictions: WynnItemRestrictions.optional(),
  dropMeta: WynnDropMeta.optional(),
  dropRestriction: WynnDropRestriction,
  requirements: WynnEquipRequirements,
  lore: z.string().optional(),
  icon: WynnItemIcon,
  base: WynnBaseStats,
  rarity:WynnItemRarity,
})

export const WapiV3ItemAccessory = z.object({
  internalName: z.string(),
  type: z.literal('accessory'),
  identified: z.boolean().optional(),
  accessoryType: z.enum([
    "ring","necklace","bracelet",
  ]),
  majorIds: z.record(z.string(), z.string()).optional(),
  restrictions: WynnItemRestrictions.optional(),
  dropMeta: WynnDropMeta.optional(),
  dropRestriction: WynnDropRestriction,
  requirements:WynnEquipRequirements,
  lore: z.string().optional(),
  icon: WynnItemIcon,
  identifications: WynnIdentifications.optional(),
  base: WynnBaseStats.optional(),
  rarity:WynnItemRarity,
}).strict()

export const WapiV3ItemIngredient = z.object({
  internalName: z.string(),
  type: z.literal('ingredient'),
  requirements: z.object({
    level: z.number(),
    skills: z.array(z.enum([
      "alchemism",
      "armouring",
      "cooking",
      "jeweling",
      "scribing",
      "tailoring",
      "weaponsmithing",
      "woodworking",
    ])),
  }),
  icon: WynnItemIcon,
  identifications: WynnIdentifications.optional(),
  tier: z.number(),
  consumableOnlyIDs: z.record(z.string(), z.number()),
  ingredientPositionModifiers: z.record(z.string(), z.number()),
  itemOnlyIDs: z.record(z.string(), z.number()),
  droppedBy: z.array(z.object({
    name: z.string(),
    coords: z.union([
      z.boolean(),
      z.array(z.number()).length(4),
      z.array(z.array(z.number()).length(4)),
    ]).nullable()
  })).optional()
}).strict()

export const WapiV3ItemMaterial = z.object({
  internalName: z.string(),
  type: z.literal('material'),
  identified: z.boolean(),
  requirements: z.object({
    level: z.number(),
  }),
  craftable: z.array(z.enum([
    'potions','food','scrolls',
    'helmets','chestplates','rings','bracelets',
    'necklaces','boots','leggings','bows','wands','spears',
    'daggers','chestplates','helmets'])),
  icon: WynnItemIcon,
  tier: z.number(),
}).strict()

export const WapiV3ItemWeapon = z.object({
  internalName: z.string(),
  type: z.literal('weapon'),
  identified: z.boolean().optional(),
  allowCraftsman: z.boolean().optional(),
  weaponType: z.enum([
    "bow","relik","wand","dagger","spear"
  ]),
  attackSpeed: z.enum([
    "super_slow", "very_slow", "slow","normal","fast", "very_fast","super_fast"
  ]),
  powderSlots: z.number().optional(),
  averageDps: z.number().optional(),
  restrictions: WynnItemRestrictions.optional(),
  dropMeta: WynnDropMeta.optional(),
  dropRestriction: WynnDropRestriction.optional(),
  requirements: WynnEquipRequirements,
  majorIds: z.record(z.string(), z.string()).optional(),
  lore: z.string().optional(),
  icon: WynnItemIcon,
  identifications: WynnIdentifications.optional(),
  base: WynnBaseStats.optional(),
  rarity:WynnItemRarity,
}).strict()


export const WapiV3ItemArmour = z.object({
  internalName: z.string(),
  type: z.literal('armour'),
  armourType: z.string(),
  armourMaterial: z.string().optional(),
  armourColor: z.string().optional(),
  identified: z.boolean().optional(),
  allowCraftsman: z.boolean().optional(),
  restrictions: WynnItemRestrictions.optional(),
  dropRestriction: WynnDropRestriction,
  dropMeta: WynnDropMeta.optional(),
  icon: WynnItemIcon.optional(),
  requirements: z.object({
    level: z.number(),
    classRequirement: z.string().optional(),
    intelligence: z.number().optional(),
    strength: z.number().optional(),
    dexterity: z.number().optional(),
    defence: z.number().optional(),
    agility: z.number().optional(),
  }),
  majorIds: z.record(z.string(), z.string()).optional(),
  powderSlots: z.number().optional(),
  lore: z.string().optional(),
  identifications: z.record(z.string(), z.union([
    z.number(),
    z.object({
      min: z.number(),
      raw: z.number(),
      max: z.number(),
    })
  ])).optional(),
  base: WynnBaseStats.optional(),
  rarity:WynnItemRarity,
}).strict()

export const WApiV3Item = z.discriminatedUnion("type",[
  WapiV3ItemMaterial,
  WapiV3ItemWeapon,
  WapiV3ItemArmour,
  WapiV3ItemIngredient,
  WapiV3ItemAccessory,
  WapiV3ItemCharm,
  WapiV3ItemTome,
  WapiV3ItemTool,
])
export const WApiV3ItemDatabase= z.record(z.string(), WApiV3Item)
